// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// This Indicator represents the DarvasDown Box.
    /// </summary>
	[Description("This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.")]
    [Gui.Design.DisplayName("DarvasDown Box")]
    public class DarvasDown : Indicator
    {
        #region Variables
		private double				boxBottom				= double.MaxValue;
		private double				boxTop			= double.MinValue;
		private bool				sellSignal			= false;
		private double				currentBarLow		= double.MaxValue;
        private double				currentBarHigh		= double.MinValue;
		private int     			savedCurrentBar		= -1;
		private bool				buySignal			= false;
		private int					startBarActBox		= 0;
        private int					state				= 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.White, PlotStyle.Square, "Lower"));
			Add(new Plot(Color.Blue, PlotStyle.Square, "Upper"));

			 
            Overlay = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			SellSignal	= false;
			BuySignal	= false;

			if (savedCurrentBar == -1) // Init
			{
				currentBarLow = Low[0];
				currentBarHigh = High[0];
				state = GetNextState();
				savedCurrentBar = CurrentBar;
			}
			else if (savedCurrentBar != CurrentBar)
			{	// Is new bar ?
				currentBarLow = Low[0];
				currentBarHigh = High[0];

				// today buy/sell signal is triggered
				if ((state == 5 && currentBarLow < boxBottom) || (state == 5 && currentBarHigh > boxTop))
				{
					if (state == 5 && currentBarLow < boxBottom)
						SellSignal	= true;
					else
						BuySignal	= true;

					state = 0;
					startBarActBox = CurrentBar;
				}

				state = GetNextState();
				// draw with today
				if (boxTop == double.MinValue)
					for (int i = CurrentBar - startBarActBox; i >= 0; i--)
						Lower.Set(i, boxBottom);
				else
					for (int i = CurrentBar - startBarActBox; i >= 0; i--)
					{
						Upper.Set(i, boxTop);
						Lower.Set(i, boxBottom);
					}

				savedCurrentBar = CurrentBar;
			}
			else
			{
				currentBarLow = Math.Min(currentBarLow, Low[0]);
				currentBarHigh = Math.Max(currentBarHigh, High[0]);

				// today buy/sell signal is triggered
				if ((state == 5 && currentBarLow < boxBottom) || (state == 5 && currentBarHigh > boxTop))
				{
					if (state == 5 && currentBarLow < boxBottom)
						SellSignal	= true;
					else
						BuySignal	= true;

					startBarActBox = CurrentBar;
					state = 0;
					state = GetNextState();
					
				}

				// draw with today
				if (boxTop == double.MinValue)
					Lower.Set(boxBottom);
				else
				{
					Lower.Set(boxBottom);
					Upper.Set(boxTop);
				}
			}

		}

        #region Properties
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public bool SellSignal
        {
            get { return sellSignal; }
			set { sellSignal = value; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Upper
        {
            get { return Values[1]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public bool BuySignal
        {
            get { return buySignal; }
			set { buySignal = value; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Lower
        {
            get { return Values[0]; }
        }
        #endregion

		#region Miscellaneous
		private int GetNextState() 
        {
			switch (state)
			{
				case 0:
					boxBottom = currentBarLow;
					boxTop = double.MinValue;
					return 1;
				
				case 1:
					if (boxBottom < currentBarLow)
						return 2;
					else
					{
						boxBottom = currentBarLow;
						return 1;
					}

				case 2:
					if (boxBottom < currentBarLow)
					{
						boxTop = currentBarHigh;
						return 3;
					}
					else
					{
						boxBottom = currentBarLow;
						return 1;
					}
				
				case 3:
					if (boxBottom < currentBarLow)
					{
						if (boxTop > currentBarHigh)
							return 4;
						else
						{
							boxTop = currentBarHigh;
							return 3;
						}
					}
					else
					{
						boxBottom = currentBarLow;
						boxTop = double.MinValue;
						return 1;
					}

				case 4:
					if (boxBottom < currentBarLow)
					{
						if (boxTop > currentBarHigh)
							return 5;
						else
						{
							boxTop = currentBarHigh;
							return 3;
						}
					}
					else
					{
						boxBottom = currentBarLow;
						boxTop = double.MinValue;
						return 1;
					}

				case 5:
					return 5;

				default:			// should not happen
					return state;
			}
        }
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DarvasDown[] cacheDarvasDown = null;

        private static DarvasDown checkDarvasDown = new DarvasDown();

        /// <summary>
        /// This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.
        /// </summary>
        /// <returns></returns>
        public DarvasDown DarvasDown()
        {
            return DarvasDown(Input);
        }

        /// <summary>
        /// This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.
        /// </summary>
        /// <returns></returns>
        public DarvasDown DarvasDown(Data.IDataSeries input)
        {

            if (cacheDarvasDown != null)
                for (int idx = 0; idx < cacheDarvasDown.Length; idx++)
                    if (cacheDarvasDown[idx].EqualsInput(input))
                        return cacheDarvasDown[idx];

            DarvasDown indicator = new DarvasDown();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            DarvasDown[] tmp = new DarvasDown[cacheDarvasDown == null ? 1 : cacheDarvasDown.Length + 1];
            if (cacheDarvasDown != null)
                cacheDarvasDown.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDarvasDown = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DarvasDown DarvasDown()
        {
            return _indicator.DarvasDown(Input);
        }

        /// <summary>
        /// This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.
        /// </summary>
        /// <returns></returns>
        public Indicator.DarvasDown DarvasDown(Data.IDataSeries input)
        {
            return _indicator.DarvasDown(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DarvasDown DarvasDown()
        {
            return _indicator.DarvasDown(Input);
        }

        /// <summary>
        /// This is the Ninja Trader implementation of Darvas Boxes, except I reversed the logic so that it works for a bear market.
        /// </summary>
        /// <returns></returns>
        public Indicator.DarvasDown DarvasDown(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DarvasDown(input);
        }

    }
}
#endregion
